/*

thargoid-carrier-script.js

Ship script for Thargoid Carriers.


Oolite
Copyright  2003-2012 Giles C Williams and contributors

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA 02110-1301, USA.


thargoid_carrier.oxp
Copyright  2006-2012 "Selezen", scripting by "Commander McLane"

This work is licensed under the Creative Commons
Attribution-Noncommercial-Share Alike 3.0 Unported License.

To view a copy of this license, visit
http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter
to Creative Commons, 171 Second Street, Suite 300, San Francisco,
California, 94105, USA.

*/

"use strict";

this.name           = "thargoid-carrier-script";
this.description    = "Ship script for Thargoid Carriers";
this.author         = "Commander McLane";
this.copyright      = " 2012 Commander McLane";
this.license        = "CC-by-nc-sa 3.0";
this.version        = "2.0";


// the functions for placing the escorts around the carrier ...
(function () {
const escortPositions =
[
    // X-shape escort pattern
    new Vector3D(-1, 0,  1),
    new Vector3D( 1, 0,  1),
    new Vector3D(-1, 0, -1),
    new Vector3D( 1, 0, -1)
];

// ... are nicked from Ahruman
this.coordinatesForEscortPosition = function (index)
{
    var highPart = 1 + (index >> 2);  // Equivalent to 1 + Math.floor(position / 4)
    var lowPart = index % 4;
    var spacing = this.ship.collisionRadius * 2.5 * highPart;
    return escortPositions[lowPart].multiply(spacing);
}
}).call(this);


this.shipSpawned = function()
{
    // on spawning of the carrier it flies in the right direction and some variables are set
    this.ship.orientation = [1, 0, 0, 0];
    this.defenseShipNumber = 0;
    this.ship.suppressArrivalReports = true;
    // the following are only necessary in-system
    if(!system.isInterstellarSpace)
    {
        this.targetStationName = this.targetStation.name;
        this.evacuationShips = 0;
        this.stationDefenders = 0;
        this.targetStationDistressTimer = new Timer(this, this.$targetStationSendDistressMessage, 2, 10);
        this.targetStationDistressTimer.stop();
    }
}

// the position of the target station becomes the carrier's destination coordinates (only called in-system)
this.$requestTargetCoordinates = function()
{
    this.ship.savedCoordinates = this.destinationCoordinates;
}

// ordinary Thargoids serve as defense ships, they don't belong to the carrier's ship group
// this is to prevent them from attempting to dock after they have killed the attacker
// also this makes their number unlimited
// in order to limit their number anyway only five are allowed at the same time
this.$launchDefenseShip = function()
{
    if(this.defenseShipNumber < 5 && system.countShipsWithPrimaryRole("thargoid", this.ship, 25600) < 8)
    {
        var defenseShip = this.ship.launchShipWithRole("thargoid");
        this.defenseShipNumber++;
    }
    // but after 120 seconds ...
    if(!this.defenseShipTimer)
    {
        this.defenseShipTimer = new Timer(this, this.$clearDefenseShips, 120);
    }
}

// ... there may be another five
this.$clearDefenseShips = function()
{
    this.defenseShipNumber = 0;
    delete this.defenseShipTimer;
}

// while approaching the target station (in-system) some reactions are triggered
this.$checkDistanceToStation = function()
{
    // a timer is started 50000 meters from the station
    if(this.ship.position.distanceTo(this.targetStation.position) < 50000)
    {
        if(!this.targetStationDistressTimer.isRunning)
        {
            this.targetStationDistressTimer.start();
        }
    }
    // 20000 meters from the station the final attack starts
    if(this.ship.position.distanceTo(this.targetStation.position) < 20000)
    {
        this.ship.reactToAIMessage("DESIRED_RANGE_ACHIEVED");
    }
}

// the timer governs the target station's reaction
this.$targetStationSendDistressMessage = function()
{
    // if it was destroyed, the timer is stopped
    if(!this.targetStation.isValid)
    {
        player.commsMessage(system.name + " control: " + this.targetStationName + " was lost. " + randomName() + " help us all! All defense forces, protect the evacuees!");
        this.targetStationDistressTimer.stop();
        return;
    }
    // the station sends one of a couple of distress messages
    this.targetStation.commsMessage(expandDescription("[thargoid_carrier_station_under_attack]"));
    // the next reaction will follow after a randomized amount of time
    this.targetStationDistressTimer.interval = 12 + (Math.random() * 10);
    // if the carrier is closer than 35000 meters, the evacuation starts by launching traders
    if(this.targetStation.position.distanceTo(this.ship.position) < 35000)
    {
        // a trader is launched, forcibly aborting all ongoing dockings
        this.targetStation.launchShipWithRole("trader", true);
        this.evacuationShips++;
        if(this.evacuationShips > 4 && this.stationDefenders < 5)
        {
            // after a couple of evacuation ships also defenses are launched
            // also aborting the ongoing dockings
            this.targetStation.launchShipWithRole("police", true);
            this.stationDefenders++;
        }
    }
    // the station-attacking ships with the plasma accelerators may go astray,
    // therefore they're set on their target again
    for(var i=0; i<system.countShipsWithRole("thargoid_plasma_accelerator"); i++)
    {
        system.shipsWithRole("thargoid_plasma_accelerator")[i].target = this.targetStation;
    }
}

// at 20000 meters from the target station, the final assault is launched
this.$launchAttackOnStation = function()
{
    if(!this.hasLaunchedAttackOnStation)
    {
        // four deadly warships with plasma accelerators are launched and set on track
        for(var i=0; i<4; i++)
        {
            var attackShip = this.ship.launchShipWithRole("thargoid_plasma_accelerator");
            attackShip.target = this.targetStation;
        }
        this.hasLaunchedAttackOnStation = true;
    }
}

this.stationLaunchedShip = function(ship)
{
    // some of the thargoids launched as defense ships can have escorts, which launch with a non-thargoid scanClass
    // in order to avoid this and the subsequent fighting amongst the thargoids, those are removed upon launch
    if(!ship.isThargoid && !ship.isPlayer)
    {
        ship.remove(true);
    }
}