/*

thargoidcarrier.js

Script for creating and handling Thargoid Carriers.


Oolite
Copyright  2003-2012 Giles C Williams and contributors

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
MA 02110-1301, USA.


thargoid_carrier.oxp
Copyright  2006-2012 "Selezen", scripting by "Commander McLane"

This work is licensed under the Creative Commons
Attribution-Noncommercial-Share Alike 3.0 Unported License.

To view a copy of this license, visit
http://creativecommons.org/licenses/by-nc-sa/3.0/ or send a letter
to Creative Commons, 171 Second Street, Suite 300, San Francisco,
California, 94105, USA.

*/

"use strict";

this.name           = "thargoidcarrier";
this.description    = "Script for creating and handling Thargoid Carriers";
this.author         = "Commander McLane";
this.copyright      = " 2012 Selezen and Commander McLane";
this.license        = "CC-by-nc-sa 3.0";
this.version        = "2.0";


this.$setUpCarriers = function()
{
    // spawning of a carrier in-system depends on the presence of secondary stations around the planet
    function $isOtherSystemStation(entity)
    {
        return entity.isStation && entity.scanClass === "CLASS_STATION" && entity.position.distanceTo(system.mainPlanet.position) < system.mainPlanet.radius * 3;
    }
    if(!system.isInterstellarSpace)
    {
        // first we find out whether there are other stations than the main station that are worth attacking
        var otherSystemStations = system.filteredEntities(this, $isOtherSystemStation, system.mainStation);
        if(otherSystemStations.length > 0)
        {
            // if yes, the station farthest away from the main station is chosen as target
            this.thargoidCarrierTargetStation = otherSystemStations[otherSystemStations.length - 1];
            // a carrier is spawned with a small probability, growing for more experienced players
            if((player.score < 128 && Math.random() < 0.02) || (player.score > 127 && player.score < 2560 && Math.random() < 0.05) || (player.score > 2559 && player.score < 6400 && Math.random() < 0.1))
            {
                // carrier is spawned halfway or further to the station
                var thargoidCarrier = system.addShips("thargoid_carrier", 1, this.thargoidCarrierTargetStation.position.multiply(0.8 - (Math.random() * 0.4)))[0];
                // the station and its position are transferred to the carrier's script
                thargoidCarrier.script.targetStation = this.thargoidCarrierTargetStation;
                thargoidCarrier.script.destinationCoordinates = this.thargoidCarrierTargetStation.position;
                // a timer is started to send warning messages to the player
                this.warningMessageTimer = new Timer(this, this.$sendWarningMessage, 6, 18);
                this.warningMessageCounter = 0;
            }
            // for ELITE players there may be two carriers
            if(player.score > 6399 && Math.random() < 0.15)
            {
                var thargoidCarrier = system.addShips("thargoid_carrier", 1, this.thargoidCarrierTargetStation.position.multiply(0.8 - (Math.random() * 0.4)))[0];
                thargoidCarrier.script.targetStation = this.thargoidCarrierTargetStation;
                thargoidCarrier.script.destinationCoordinates = this.thargoidCarrierTargetStation.position;
                this.warningMessageTimer = new Timer(this, this.$sendWarningMessage, 6, 18);
                this.warningMessageCounter = 0;
                // 20% of the time a second carrier is spawned
                if(Math.random() < 0.2)
                {
                    var thargoidCarrier = system.addShips("thargoid_carrier", 1, this.thargoidCarrierTargetStation.position.multiply(0.8 - (Math.random() * 0.4)))[0];
                    thargoidCarrier.script.targetStation = this.thargoidCarrierTargetStation;
                    thargoidCarrier.script.destinationCoordinates = this.thargoidCarrierTargetStation.position;

                }
            }
        }
    }
    // in interstellar space ...
    else
    {
        // ... there is a bigger chance to meet a carrier
        if((player.score < 128 && Math.random() < 0.2) || (player.score > 127 && player.score < 512 && Math.random() < 0.25) || (player.score > 511 && player.score < 6400 && Math.random() < 0.3) || (player.score > 6399 && Math.random() < 0.5))
        {
            system.addShips("thargoid_carrier", 1);
            if(player.score > 6399 && Math.random() < 0.3) system.addShips("thargoid_carrier", 1);
        }
    }
}

this.$sendWarningMessage = function()
{
    // if the carrier has been destroyed there is a message and the timer is stopped
    if(system.countShipsWithRole("thargoid_carrier") === 0 && this.thargoidCarrierTargetStation.isValid)
    {
        player.commsMessage(system.name + " control: Thargoid Carrier defeated. Thank " + randomName() + "! All ships, beware of remaining Thargoids and Robot Fighters.");
        this.warningMessageTimer.stop();
        return;
    }
    // we progress to the next warning message
    this.warningMessageCounter++;
    // and set a random inteval until the following message
    this.warningMessageTimer.interval = 12 + (Math.random() * 13);
    // a total of four messages, becoming progressively more concrete and urgent
    switch(this.warningMessageCounter)
    {
        case 1:
            player.commsMessage("This is " + system.name + " control: A Thargoid Carrier was detected approaching the planet. Civilian traffic beware.");
            break;
            
        case 2:
            player.commsMessage(system.name + " control: Warning! Thargoid Carrier confirmed in the system. All defenses, stand by!");
            break;
            
        case 3:
            player.commsMessage(system.name + " control: Thargoid Carrier seems to be heading towards the " + this.thargoidCarrierTargetStation.name + ".");
            break;
            
        case 4:
            player.commsMessage(system.name + " control: Thargoid Carrier confirmed heading towards the " + this.thargoidCarrierTargetStation.name + ". All defenses, engage! Engage!");
            break;
    }
}

this.shipExitedWitchspace = function()
{
    // carriers are spawned
    this.$setUpCarriers();
}

this.shipWillDockWithStation = function(station)
{
    // no docking with the carrier
    if(station.name === "Thargoid Carrier")
    {
        player.ship.launch();
    }
}

this.shipWillEnterWitchspace = function()
{
    // cleanup on leaving the system
    if(this.warningMessageTimer)
    {
        this.warningMessageTimer.stop();
        delete this.warningMessageTimer;
    }
}
